/*
 * Compute the speed of various tasks
 */

#include "config.h"

#include "bloom.h"

#include <stdio.h>
#include <sys/types.h>
#include <openssl/hmac.h>
#include <openssl/rand.h>
#include <string.h>
#include <cstring>
#include "aftimer.h"
#include "crc64.h"
#include "adler32.h"


int adler32_update(EVP_MD_CTX *ctx,const void *data,size_t count)
{
    printf("adler32_update(count=%zd)\n",count);
    return 0;
}

int adler32_final(EVP_MD_CTX *ctx,unsigned char *md)
{
    printf("adler32_final\n");
    return(0);
}

int touch(u_char *in,size_t insize)
{
    int t=0;
    for(u_int i=0;i<insize;i++){
	t += in[i];
    }
    return insize;
}

void hashtest(size_t insize)
{
    const char *tests[] = {"BYTESUM","FAST32","Adler32","CRC64","MD5","SHA1","SHA256",0};
    u_char *in = (u_char *)alloca(insize);
    memset((void *)in,0,insize);

    struct env_md_st ea;

    memset(&ea,0,sizeof(ea));
    ea.md_size = 4;
    ea.update = adler32_update;
    ea.final = adler32_final;
    ea.block_size = 1;
    ea.ctx_size = 1;


    int COUNT=1000000*512/insize+25000;

    printf("Block size: %lu bytes\n",insize);
    printf("Loop size: %d hashes\n",COUNT);
    printf("-----------------------------\n");
    for(int j=0;tests[j];j++){
	printf("Hash: %7s ...",tests[j]);
	fflush(stdout);
	aftimer t;
	unsigned int r = 0;

	t.start();
	for(int i=0;i<COUNT;i++){
	    u_char out[EVP_MAX_MD_SIZE];
	    u_int outlen;

	    memset(out,0,EVP_MAX_MD_SIZE);

	    outlen = sizeof(out);
	    in[0] = i;			// change the input
	    switch(j){
	    case 0:			r = touch(in,insize);			  break;
	    case 1:                     /*r = fast32(in,insize);*/		          break;
	    case 2: /* Adler 32 */      r = av_adler32_update(1,in,insize);	          break;
	    case 3: /* CRC64 */         crc64(in,insize,out);		                  break;
	    case 4: /* MD5 */		EVP_Digest(in,insize,out,&outlen,EVP_md5(),0);    break;
	    case 5: /* SHA1 */		EVP_Digest(in,insize,out,&outlen,EVP_sha1(),0);   break;
	    case 6: /* SHA256 */        EVP_Digest(in,insize,out,&outlen,EVP_sha256(),0); break;
	    }
	    if(j>2) r = *(unsigned int *)&out[0];
	}
	t.stop();
	printf("time: %8.2f sec  ; %8.1f Khashes/sec; %8.1f Mb/sec r=%x\n",
	       t.elapsed_seconds(),
	       ((double)COUNT)/t.elapsed_seconds()/1000.0,
	       ((double)COUNT)*insize/t.elapsed_seconds()/1000000.0,
	       r
	       );
    }

    printf("---------------\n");
}


int main(int argc,char **argv)
{
    hashtest(512);
    hashtest(4096);
    hashtest(65536);

    return(0);
}

